@@ -180,7 +180,7 @@ gda_column_get_type (void)
 {
 	static GType type = 0;
 	
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaColumnClass),
 			(GBaseInitFunc) NULL,
