@@ -53,6 +53,9 @@ typedef enum {
 	GDA_SERVER_OPERATION_CREATE_INDEX,
 	GDA_SERVER_OPERATION_DROP_INDEX,
 
+	GDA_SERVER_OPERATION_CREATE_VIEW,
+	GDA_SERVER_OPERATION_DROP_VIEW,
+
 	GDA_SERVER_OPERATION_NB
 } GdaServerOperationType;
 
@@ -77,10 +80,10 @@ typedef struct _GdaServerOperationNode {
 	GdaServerOperationNodeType    type;
 	GdaServerOperationNodeStatus  status;
 	
-	GdaParameterList             *plist;
+	GdaSet                       *plist;
 	GdaDataModel                 *model;
 	GdaColumn                    *column;
-	GdaParameter                 *param; 
+	GdaHolder                    *param; 
 	gpointer                      priv;
 } GdaServerOperationNode;
 
@@ -98,7 +101,7 @@ struct _GdaServerOperationClass {
 
 };
 
-GType                      gda_server_operation_get_type                (void);
+GType                      gda_server_operation_get_type                (void) G_GNUC_CONST;
 GdaServerOperation        *gda_server_operation_new                     (GdaServerOperationType op_type, const gchar *xml_file);
 GdaServerOperationType     gda_server_operation_get_op_type             (GdaServerOperation *op);
 const gchar               *gda_server_operation_op_type_to_string       (GdaServerOperationType type);
