@@ -1,6 +1,6 @@
 /* 
  * GDA common library
- * Copyright (C) 2007 The GNOME Foundation.
+ * Copyright (C) 2007 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -59,15 +59,13 @@ static void gda_vprovider_data_model_get_property (GObject *object,
 static GObjectClass  *parent_class = NULL;
 
 static GdaConnection *gda_vprovider_data_model_create_connection (GdaServerProvider *provider);
-static gboolean       gda_vprovider_data_model_open_connection (GdaServerProvider *provider,
-								GdaConnection *cnc,
-								GdaQuarkList *params,
-								const gchar *username,
-								const gchar *password);
+static gboolean       gda_vprovider_data_model_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
+								GdaQuarkList *params, GdaQuarkList *auth,
+								guint *task_id, GdaServerProviderAsyncCallback async_cb, 
+								gpointer cb_data);
 static gboolean       gda_vprovider_data_model_close_connection (GdaServerProvider *provider,
 								 GdaConnection *cnc);
-static GdaServerProviderInfo *gda_vprovider_data_model_get_info (GdaServerProvider *provider,
-								 GdaConnection *cnc);
+static const gchar   *gda_vprovider_data_model_get_name (GdaServerProvider *provider);
 
 /*
  * GdaVproviderDataModel class implementation
@@ -85,7 +83,13 @@ gda_vprovider_data_model_class_init (GdaVproviderDataModelClass *klass)
 	server_class->open_connection = gda_vprovider_data_model_open_connection;
 	server_class->close_connection = gda_vprovider_data_model_close_connection;
 
-	server_class->get_info = gda_vprovider_data_model_get_info;
+	server_class->get_name = gda_vprovider_data_model_get_name;
+
+	/* explicitely unimplement the DDL queries */
+	server_class->supports_operation = NULL;
+        server_class->create_operation = NULL;
+        server_class->render_operation = NULL;
+        server_class->perform_operation = NULL;
 
 	/* Properties */
         object_class->set_property = gda_vprovider_data_model_set_property;
@@ -244,15 +248,19 @@ gda_vprovider_data_model_create_connection (GdaServerProvider *provider)
 
 static gboolean
 gda_vprovider_data_model_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
-					  GdaQuarkList *params,
-					  const gchar *username,
-					  const gchar *password)
+					  GdaQuarkList *params, GdaQuarkList *auth,
+					  guint *task_id, GdaServerProviderAsyncCallback async_cb, gpointer cb_data)
 {
 	GdaQuarkList *m_params;
 
 	g_return_val_if_fail (GDA_IS_VPROVIDER_DATA_MODEL (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_VCONNECTION_DATA_MODEL (cnc), FALSE);
 
+	if (async_cb) {
+		gda_connection_add_event_string (cnc, _("Provider does not support asynchronous connection open"));
+                return FALSE;
+	}
+
 	if (params) {
 		m_params = gda_quark_list_copy (params);
 		gda_quark_list_add_from_string (m_params, "_IS_VIRTUAL=TRUE;LOAD_GDA_FUNCTIONS=TRUE", TRUE);
@@ -261,16 +269,15 @@ gda_vprovider_data_model_open_connection (GdaServerProvider *provider, GdaConnec
 		params = gda_quark_list_new_from_string ("_IS_VIRTUAL=TRUE;LOAD_GDA_FUNCTIONS=TRUE");
 
 	if (! GDA_SERVER_PROVIDER_CLASS (parent_class)->open_connection (GDA_SERVER_PROVIDER (provider), cnc, m_params,
-									 NULL, NULL)) {
+									 auth, NULL, NULL, NULL)) {
 		gda_quark_list_free (m_params);
 		return FALSE;
 	}
 	gda_quark_list_free (m_params);
 
-	SQLITEcnc *scnc;
-	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
+	SqliteConnectionData *scnc;
+	scnc = (SqliteConnectionData*) gda_connection_internal_get_provider_data ((GdaConnection *) cnc);
 	if (!scnc) {
-		gda_connection_add_event_string (cnc, _("Invalid SQLite handle"));
 		gda_connection_close_no_warning (cnc);
 
 		return FALSE;
@@ -304,21 +311,10 @@ gda_vprovider_data_model_close_connection (GdaServerProvider *provider, GdaConne
 	return GDA_SERVER_PROVIDER_CLASS (parent_class)->close_connection (GDA_SERVER_PROVIDER (provider), cnc);
 }
 
-static GdaServerProviderInfo *
-gda_vprovider_data_model_get_info (GdaServerProvider *provider, GdaConnection *cnc)
+static const gchar *
+gda_vprovider_data_model_get_name (GdaServerProvider *provider)
 {
-	static GdaServerProviderInfo info;
-        static gboolean init_done = FALSE;
-
-        if (!init_done) {
-                GdaServerProviderInfo *vinfo;
-                vinfo = GDA_SERVER_PROVIDER_CLASS (parent_class)->get_info (provider, cnc);
-                info = *vinfo;
-                info.provider_name = "Virtual";
-                init_done = TRUE;
-        }
-
-        return &info;
+	return "Virtual";
 }
 
 /* module implementation */
@@ -376,7 +372,6 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 		else {
 			if (gda_data_model_get_access_flags (td->spec->data_model) & GDA_DATA_MODEL_ACCESS_RANDOM)
 				proxy = g_object_new (GDA_TYPE_DATA_PROXY, 
-						      "dict", gda_object_get_dict (GDA_OBJECT (td->spec->data_model)), 
 						      "model", td->spec->data_model, 
 						      "sample_size", 0, NULL);
 			else {
@@ -571,7 +566,7 @@ virtualOpen (sqlite3_vtab *pVTab, sqlite3_vtab_cursor **ppCursor)
 	virtual_table_manage_real_data_model (vtable);
 
 	cursor = g_new0 (VirtualCursor, 1);
-	cursor->iter = gda_data_model_iter_new (GDA_DATA_MODEL (vtable->proxy));
+	cursor->iter = gda_data_model_create_iter (GDA_DATA_MODEL (vtable->proxy));
 	cursor->ncols = gda_data_model_get_n_columns (GDA_DATA_MODEL (vtable->td->real_model));
 	*ppCursor = &(cursor->base);
 	return SQLITE_OK;
@@ -625,7 +620,7 @@ virtualColumn (sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int i)
 
 	TRACE ();
 
-	GdaParameter *param;
+	GdaHolder *param;
 	
 	if (i == cursor->ncols) {
 		/* private hidden column, which returns the row number */
@@ -633,14 +628,14 @@ virtualColumn (sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int i)
 		return SQLITE_OK;
 	}
 
-	param = gda_data_model_iter_get_param_for_column (cursor->iter, i);
+	param = gda_data_model_iter_get_holder_for_field (cursor->iter, i);
 	if (!param) {
 		sqlite3_result_error (ctx, _("Column not found"), -1);
 		return SQLITE_EMPTY;
 	}
 	else {
 		const GValue *value;
-		value = gda_parameter_get_value (param);
+		value = gda_holder_get_value (param);
 
 		if (!value || gda_value_is_null (value))
 			sqlite3_result_null (ctx);
@@ -739,7 +734,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 
 			/*g_print ("%d => %s\n", i, sqlite3_value_text (apData [i]));*/
 			type = gda_column_get_g_type (gda_data_model_describe_column ((GdaDataModel*) vtable->proxy, i - 2));
-			value = gda_value_new_from_string (sqlite3_value_text (apData [i]), type);
+			value = gda_value_new_from_string ((const gchar*) sqlite3_value_text (apData [i]), type);
 			res = gda_data_model_set_value_at ((GdaDataModel*) vtable->proxy, i - 2, rowid, value, &error);
 			gda_value_free (value);
 			if (!res) {
@@ -768,7 +763,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 			GValue *value;
 			type = gda_column_get_g_type (gda_data_model_describe_column ((GdaDataModel*) vtable->proxy, i - 2));
 			if (sqlite3_value_text (apData [i]))
-				value = gda_value_new_from_string (sqlite3_value_text (apData [i]), type);
+				value = gda_value_new_from_string ((const gchar*) sqlite3_value_text (apData [i]), type);
 			else
 				value = gda_value_new_null ();
 			/*g_print ("TXT #%s# => value %p (type=%s)\n", sqlite3_value_text (apData [i]), value,
